var vows = require('vows')
var assert = require('assert')
var Runtime = require('fire').Runtime
var runTestServer = require('./helper.js').runTestServer
var fire = require('fire')
var fireHttp = require('../index.js')
var path = require('path')
var qs = require('querystring');

vows.describe('Http Get').addBatch({
	"When I perfom a GET on a URL that returns 'Hello World' and 200 OK": {
		topic: function() {
			var self = this
			runTestServer.call(this,"t.1.get", function(finishWithServer) {
				var runtime = new Runtime()
				runtime.setBaseDir(path.dirname(module.filename))
				runtime.loadModuleInstance(fireHttp, 'fire-http')
				
				runtime.registerWellKnownExpressionDefinition({
					name: "TestExp",
					json: {
						"@Http(GET)": {
							uri: "http://127.0.0.1:3600/"
						}
					}
				})
				runtime.load(function(initError) {
					if(initError) {
						finishWithServer(initError, null)
					} else {
						
						var expDef = runtime.getExpressionDefinition('TestExp')
						var exp = new(expDef.implementation)
						exp.resultCallback = function(res) {
							finishWithServer(null, res)
						};
						exp.errorCallback = function(err) {
							finishWithServer(err, null)
						}
						exp.runtime = runtime
						exp.run()
					}
				})
			}) // runTestServer
		},
		"There should not be any error on the server side": function(res){
			assert.isEmpty(res.stderr)
		},
		"There should not be any error on the client side": function(res){
			assert.isNull(res.testErr)
		},
		"The response body should be 'Hello World'": function(res){
			assert.strictEqual(res.testRes.body, "Hello World")
		},
		"The response status should be 200": function(res){
			assert.equal(res.testRes.response.statusCode, 200)
		}
	},
	"When I perfom a GET on a URL that returns 'Hello World' and 201 Created": {
		topic: function() {
			var self = this
			runTestServer.call(this,"t.2.get", function(finishWithServer) {
				var runtime = new Runtime()
				runtime.setBaseDir(path.dirname(module.filename))
				runtime.loadModuleInstance(fireHttp, 'fire-http')
				
				runtime.registerWellKnownExpressionDefinition({
					name: "TestExp",
					json: {
						"@Http(GET)": {
							uri: "http://127.0.0.1:3601/"
						}
					}
				})
				runtime.load(function(initError) {
					if(initError) {
						finishWithServer(initError, null)
					} else {
						
						var expDef = runtime.getExpressionDefinition('TestExp')
						var exp = new(expDef.implementation)
						exp.resultCallback = function(res) {
							finishWithServer(null, res)
						};
						exp.errorCallback = function(err) {
							finishWithServer(err, null)
						}
						exp.runtime = runtime
						exp.run()
					}
				})
			}) // runTestServer
		},
		"There should not be any error on the server side": function(res){
			assert.isEmpty(res.stderr)
		},
		"There should not be any error on the client side": function(res){
			assert.isNull(res.testErr)
		},
		"The response body should be 'Hello World'": function(res){
			assert.strictEqual(res.testRes.body, "Hello World")
		},
		"The response status should be 201": function(res){
			assert.equal(res.testRes.response.statusCode, 201)
		}
	}
}).export(module);


vows.describe('Http Post').addBatch({
	"When I perfom a POST using the hint on a URL with form data and the server returns the same form in JSON and 200 OK": {
		topic: function() {
			var self = this
			runTestServer.call(this,"t.3.post", function(finishWithServer) {
				var runtime = new Runtime()
				runtime.setBaseDir(path.dirname(module.filename))
				runtime.loadModuleInstance(fireHttp, 'fire-http')
				
				runtime.registerWellKnownExpressionDefinition({
					name: "TestExp",
					json: {
						"@Http(POST)": {
							uri: "http://127.0.0.1:3602/",
							body: qs.stringify({name:"myName"})
						}
					}
				})
				runtime.load(function(initError) {
					if(initError) {
						finishWithServer(initError, null)
					} else {
						
						var expDef = runtime.getExpressionDefinition('TestExp')
						var exp = new(expDef.implementation)
						exp.resultCallback = function(res) {
							finishWithServer(null, res)
						};
						exp.errorCallback = function(err) {
							finishWithServer(err, null)
						}
						exp.runtime = runtime
						exp.run()
					}
				})
			}) // runTestServer
		},
		"There should not be any error on the server side": function(res){
			assert.isEmpty(res.stderr)
		},
		"There should not be any error on the client side": function(res){
			assert.isNull(res.testErr)
		},
		"The response body should be the JSON document": function(res){
			assert.deepEqual(JSON.parse(res.testRes.body), {name:"myName"})
		},
		"The response status should be 200": function(res){
			assert.equal(res.testRes.response.statusCode, 200)
		}
	},
	"When I perfom a using the input on a URL with form data and the server returns the same form in JSON and 200 OK": {
		topic: function() {
			var self = this
			runTestServer.call(this,"t.4.post", function(finishWithServer) {
				var runtime = new Runtime()
				runtime.setBaseDir(path.dirname(module.filename))
				runtime.loadModuleInstance(fireHttp, 'fire-http')
				
				runtime.registerWellKnownExpressionDefinition({
					name: "TestExp",
					json: {
						"@Http": {
							method: "POST",
							uri: "http://127.0.0.1:3603/",
							body: qs.stringify({name:"myName"})
						}
					}
				})
				runtime.load(function(initError) {
					if(initError) {
						finishWithServer(initError, null)
					} else {
						
						var expDef = runtime.getExpressionDefinition('TestExp')
						var exp = new(expDef.implementation)
						exp.resultCallback = function(res) {
							finishWithServer(null, res)
						};
						exp.errorCallback = function(err) {
							finishWithServer(err, null)
						}
						exp.runtime = runtime
						exp.run()
					}
				})
			}) // runTestServer
		},
		"There should not be any error on the server side": function(res){
			assert.isEmpty(res.stderr)
		},
		"There should not be any error on the client side": function(res){
			assert.isNull(res.testErr)
		},
		"The response body should be the JSON document": function(res){
			assert.deepEqual(JSON.parse(res.testRes.body), {name:"myName"})
		},
		"The response status should be 200": function(res){
			assert.equal(res.testRes.response.statusCode, 200)
		}
	}
}).export(module);


vows.describe('Http Headers').addBatch({
	"When I perfom a GET on a URL with a custom header and the server responds with the value of the header": {
		topic: function() {
			var self = this
			runTestServer.call(this,"t.5.header", function(finishWithServer) {
				var runtime = new Runtime()
				runtime.setBaseDir(path.dirname(module.filename))
				runtime.loadModuleInstance(fireHttp, 'fire-http')
				
				runtime.registerWellKnownExpressionDefinition({
					name: "TestExp",
					json: {
						"@Http(GET)": {
							uri: "http://127.0.0.1:3604/",
							headers: {
								"CustomHeader": "Stuff"
							}
						}
					}
				})
				runtime.load(function(initError) {
					if(initError) {
						finishWithServer(initError, null)
					} else {
						
						var expDef = runtime.getExpressionDefinition('TestExp')
						var exp = new(expDef.implementation)
						exp.resultCallback = function(res) {
							finishWithServer(null, res)
						};
						exp.errorCallback = function(err) {
							finishWithServer(err, null)
						}
						exp.runtime = runtime
						exp.run()
					}
				})
			}) // runTestServer
		},
		"There should not be any error on the server side": function(res){
			assert.isEmpty(res.stderr)
		},
		"There should not be any error on the client side": function(res){
			assert.isNull(res.testErr)
		},
		"The response body should be the composed value with the header": function(res){
			assert.deepEqual(res.testRes.body, "Custom Header Value: Stuff")
		},
		"The response status should be 200": function(res){
			assert.equal(res.testRes.response.statusCode, 200)
		}
	}
}).export(module);

vows.describe('Http Autorization').addBatch({
	"When I use Autorization expression to encode user and password with 'Basic' scheme in the hint": {
		topic: function() {
			var self = this
			var runtime = new Runtime()
			runtime.setBaseDir(path.dirname(module.filename))
			runtime.loadModuleInstance(fireHttp, 'fire-http')
			
			runtime.registerWellKnownExpressionDefinition({
				name: "TestExp",
				json: {
					"@Http.Headers.Authorization(Basic)": {
						user: "Chuck",
						password: "SuperChuck"
					}
				}
			})
			runtime.load(function(initError) {
				if(initError) {
					self.callback(initError, null)
				} else {
					
					var expDef = runtime.getExpressionDefinition('TestExp')
					var exp = new(expDef.implementation)
					exp.resultCallback = function(res) {
						self.callback(null, res)
					};
					exp.errorCallback = function(err) {
						self.callback(err, null)
					}
					exp.runtime = runtime
					exp.run()
				}
			})
		},
		"The result should be the encoded Authorization header": function(res){
			assert.equal(res, "Basic Q2h1Y2s6U3VwZXJDaHVjaw==")
		}
	},
	"When I use Autorization expression to encode user and password with 'Custom' scheme in the input": {
		topic: function() {
			var self = this
			var runtime = new Runtime()
			runtime.setBaseDir(path.dirname(module.filename))
			runtime.loadModuleInstance(fireHttp, 'fire-http')
			
			runtime.registerWellKnownExpressionDefinition({
				name: "TestExp",
				json: {
					"@Http.Headers.Authorization": {
						scheme: "Custom",
						user: "Chuck",
						password: "SuperChuck"
					}
				}
			})
			runtime.load(function(initError) {
				if(initError) {
					self.callback(initError, null)
				} else {
					
					var expDef = runtime.getExpressionDefinition('TestExp')
					var exp = new(expDef.implementation)
					exp.resultCallback = function(res) {
						self.callback(null, res)
					};
					exp.errorCallback = function(err) {
						self.callback(err, null)
					}
					exp.runtime = runtime
					exp.run()
				}
			})
		},
		"The result should be the encoded Authorization header": function(res){
			assert.equal(res, "Custom Q2h1Y2s6U3VwZXJDaHVjaw==")
		}
	}
}).export(module);

